#include "stdafx.h"
#include "Algorithm.h"

#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/kruskal_min_spanning_tree.hpp>

using namespace boost;
using namespace std;

//
// Kruskal's algorithm for finding minimum spanning tree
//
void Algorithm::Kruskal( Network &network, vector< Link > &mst )
{
    typedef adjacency_list < vecS, 
							 vecS, 
							 undirectedS,
							 no_property, 
							 property < edge_weight_t, double > 
						   > Graph;

	typedef graph_traits < Graph >::edge_descriptor   Edge;
	typedef graph_traits < Graph >::vertex_descriptor Vertex;
	typedef std::pair<int, int> E;

	const int num_nodes = network.GetNodeCount();
	const int num_links = network.GetLinkCount();	
	

	Graph g( num_nodes );

	property_map< Graph, edge_weight_t >::type weightmap = 
		get( edge_weight, g );

	for ( size_t j = 0; 
		  j < (size_t) num_links; 
		  ++j ) 
	{
		Edge e; 
		bool inserted;
		tie( e, inserted ) = add_edge( network.GetLinkStartNode( (int) j ), 
									   network.GetLinkEndNode( (int)j ), 
									   g );

		

		weightmap[ e ] = network.GetLinkWeight( (int) j );
	}

  property_map < Graph, edge_weight_t >::type weight = 
	  get( edge_weight, g );

  std::vector < Edge > spanning_tree;

  kruskal_minimum_spanning_tree( g, std::back_inserter( spanning_tree ) );

  std::cout << "Print the edges in the MST:" << std::endl;

  for ( vector<Edge>::iterator ei = spanning_tree.begin();
        ei != spanning_tree.end();
		++ei ) 
  {		
	  Link l( (int) source(*ei, g), 
		      (int) target(*ei, g), 
			  (double) weight[*ei] );

	  mst.push_back( l );
  }	
}
