// Graph4View.cpp : implementation of the CGraph4View class
//

#include "stdafx.h"
#include "Graph4.h"

#include "Graph4Doc.h"
#include "Graph4View.h"

#include "LinkProperties.h"
#include "NodeProperties.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CGraph4View

IMPLEMENT_DYNCREATE(CGraph4View, CView)

BEGIN_MESSAGE_MAP(CGraph4View, CView)
	// Standard printing commands	
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_COMMAND(ID_ALGORITHM_KRUSKAL, OnAlgorithmKruskal)
	ON_WM_RBUTTONDOWN()
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID__DELETE_LINK, OnDeleteLink)
	ON_UPDATE_COMMAND_UI(ID__DELETE_LINK, OnDeleteLinkUpdate)
	ON_COMMAND(ID_LINK_PROPS, OnLinkProps)
	ON_COMMAND(ID_ALGORITHM_RESTORE, OnAlgorithmRestore)
	ON_WM_MOUSEMOVE()
	ON_COMMAND(ID_NODEPROPERTIES, OnNodeProperties)
END_MESSAGE_MAP()

// CGraph4View construction/destruction

CGraph4View::CGraph4View()
{
	NodeHeld = FALSE;
}

CGraph4View::~CGraph4View()
{
}

BOOL CGraph4View::PreCreateWindow( CREATESTRUCT& cs )
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	network.SetPwnd( this );		
	return CView::PreCreateWindow(cs);
}

// CGraph4View drawing

void CGraph4View::OnDraw(CDC* /*pDC*/)
{
	CGraph4Doc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	if (!pDoc)
		return;

	// TODO: add draw code for native data here
	PaintNodeset();
	DrawLinkset();
}


// CGraph4View printing

BOOL CGraph4View::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CGraph4View::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CGraph4View::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}


// CGraph4View diagnostics

#ifdef _DEBUG
void CGraph4View::AssertValid() const
{
	CView::AssertValid();
}

void CGraph4View::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CGraph4Doc* CGraph4View::GetDocument() const // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CGraph4Doc)));
	return (CGraph4Doc*)m_pDocument;
}
#endif //_DEBUG


//
// Mouse handling function for left button up
//
void CGraph4View::OnLButtonDown(UINT nFlags, CPoint point)
{
	int nodeID = -1;

	// Id node does not yet exist, add it
	if ( !network.NodeExists( point, nodeID ) )
	{
		network.AddNode( point );
	}
	else
	{
		// ... otherwise flag that left mouse button held down
		LBdownNodeID = nodeID;
		NodeHeld = TRUE;
		startNodePoint = point;
		currLinePoint = point;
	}

	UpdateData( FALSE );
	RedrawWindow( 0, 0, RDW_INVALIDATE );

	CView::OnLButtonDown(nFlags, point);
}


// 
// Paint the set of nodes
//
void CGraph4View::PaintNodeset()
{
	network.PaintNodes();
}


//
// Draw the links
//
void CGraph4View::DrawLinkset()
{
	network.DrawLinks( network.links, FALSE );	
}


//
// Mouse handling function for left button up
//
void CGraph4View::OnLButtonUp( UINT nFlags, CPoint point )
{
	int nodeID = -1;

	if ( network.NodeExists( point, nodeID ) )
	{		
		LBupNodeID = nodeID;
	}

	if ( NodeHeld )
	{
		int startID  = LBdownNodeID;
		int targetID = LBupNodeID;
		NodeHeld     = FALSE;

		CClientDC dc( this );
		InvertLine( &dc, startNodePoint, currLinePoint );

		if ( !network.LinkExists( startID, targetID ) && 
			  startID != targetID )
		{
			network.AddLink( startID, targetID );
		}
	}

	UpdateData( FALSE );
	RedrawWindow( 0, 0, RDW_INVALIDATE );

	CView::OnLButtonUp(nFlags, point);
}


//
// Use boost version of Kruskal to find minimal spanning tree
//
void CGraph4View::OnAlgorithmKruskal()
{
	network.ResetLinkColours();

	// Use Boost library to get vector list contain MST links
	// colour them black
	vector< Link > mspantree;	
	algorithm.Kruskal( network, mspantree );		
	network.ColourLinks( mspantree, 1 );

	// For each matching link in change the colour
	vector< Link >::iterator mst_it;
	for ( mst_it  = mspantree.begin();
		  mst_it != mspantree.end();
		  mst_it++ )
	{		
		Link l = (*mst_it);
	}

	
	network.DrawLinks( mspantree, TRUE );
	network.PaintNodes();
	

	UpdateData( FALSE );

	RedrawWindow( NULL, NULL, RDW_INVALIDATE | RDW_ERASE );
}


//
// Mouse handling function for left button up
//
void CGraph4View::OnRButtonDown(UINT nFlags, CPoint point)
{
	CMenu popupMenu;
	curr_point = point;
	
	// Establish if node lives near to mouse click point
	int nodeID = -1;

	// Id node does not yet exist, add it
	if ( network.NodeExists( point, nodeID ) )
	{
		network.GetNodeData( nodeID, curr_point, curr_node_label );	
		curr_node = nodeID;
		popupMenu.LoadMenu( IDR_MENU_NODEPROPS );
		CMenu* subMenu = popupMenu.GetSubMenu( 0 );
		ClientToScreen( &point );

		subMenu->TrackPopupMenu( 0, 
								point.x, 
								point.y,
								AfxGetMainWnd(), 
								NULL );
	}
	else
	{
		BOOL LinkExists = network.GetLinkFromPoint( curr_point, 
												curr_start, 
												curr_end,
												curr_weight );
		if ( LinkExists )
		{		
			popupMenu.LoadMenu( IDR_MENU_PROPS );
			CMenu* subMenu = popupMenu.GetSubMenu( 0 );
			ClientToScreen( &point );

			subMenu->TrackPopupMenu( 0, 
									point.x, 
									point.y,
									AfxGetMainWnd(), 
									NULL );
		}	
	}
	
	CView::OnRButtonDown(nFlags, point);
}


//
// Message handling function for when user clicks the right mouse button in 
// the client area, opening the context menu. 
//
void CGraph4View::OnContextMenu(CWnd* /*pWnd*/, CPoint /*point*/)
{
	CMenu mnuPopupSubmit;
	mnuPopupSubmit.LoadMenu( IDR_MENU_PROPS );
}

//
// Operation to remove a single network link
//
void CGraph4View::OnDeleteLink()
{
	network.RemoveLink( curr_start, curr_end );				
}


//
// Refresh the screen client area when link deleted
//
void CGraph4View::OnDeleteLinkUpdate(CCmdUI *pCmdUI)
{	
	UpdateData( FALSE );		
	RedrawWindow( NULL, NULL, RDW_INVALIDATE );	
}

//
// Display the link propertis
//
void CGraph4View::OnLinkProps()
{
	int start    = -1;
	int target   = -1;
	double weight   = 1.0;
	double capacity = 64.0;

	if ( LinkExists( curr_point, start, target, weight ) )
	{
		CLinkProperties linkprop;
		linkprop.SetWeight( weight );		
		linkprop.SetNodes( start, target );
		linkprop.DoModal();
		
		SetLinkWeight( start, target, linkprop.GetWeight() );	
	}		
}



//
// Display the node proprty dialog
//
void CGraph4View::OnNodeProperties()
{
	CNodeProperties nodeprop;
	nodeprop.SetPoint( curr_point );
	nodeprop.SetLabel( curr_node_label );
	nodeprop.DoModal();

	SetNodeData( curr_node, nodeprop.GetLabel() );
	UpdateData( FALSE );		
	RedrawWindow( NULL, NULL, RDW_INVALIDATE );	
}


//
// Set the node properties
//
void CGraph4View::SetNodeData( int node, CString label )
{
	network.SetNodeData( node, label );
}

//
// Get the link properties, if this link exists
//
BOOL CGraph4View::LinkExists( CPoint point, 
							 int& start, 
							 int& target,
							 double& weight )
{
	// Establish if a links exists somewhere near this mouse point
	// First of all get the distance using basic geometry
	// For each link get the (x,y) coordinates:
	return network.GetLinkFromPoint( point, start, target, weight );	
}


//
// Set the link weight
//
void CGraph4View::SetLinkWeight( int start, int target, CString weight )
{
	network.SetLinkWeight( start, target, weight );
}


//
// Restore and display your original network
//
void CGraph4View::OnAlgorithmRestore()
{	
	network.ColourLinks( network.links, 1 );	
	UpdateData( FALSE );
	RedrawWindow( NULL, NULL, RDW_INVALIDATE );
}


//
// Message handling function for mouse movements
//
void CGraph4View::OnMouseMove( UINT nFlags, CPoint point )
{
	if ( NodeHeld )
	{
		CClientDC dc( this );
		InvertLine( &dc, startNodePoint, currLinePoint );
		InvertLine( &dc, startNodePoint, point );
		currLinePoint = point;
	}	
	
	CView::OnMouseMove(nFlags, point);
}


//
// Draw a line from start to end node
//
void CGraph4View::InvertLine( CDC *pdc, CPoint ptFrom, CPoint ptTo )
{
	int nOldMode = pdc->SetROP2( R2_NOT );
	pdc->MoveTo( ptFrom );
	pdc->LineTo( ptTo );
	pdc->SetROP2( nOldMode );
}