// Graph4View.h : interface of the CGraph4View class
//
#pragma once

#include "Network.h"
#include "Algorithm.h"

class CGraph4View : public CView
{
protected: // create from serialization only
	CGraph4View();
	DECLARE_DYNCREATE(CGraph4View)

// Attributes
public:
	CGraph4Doc* GetDocument() const;

protected:
	void PaintNodeset();
	void DrawLinkset();

	BOOL LinkExists( CPoint point, 
					 int& start, 
					 int& target,
					 double& weight );

	void SetLinkWeight( int start, int target, CString weight );
	void SetNodeData( int node, CString label );

// Operations
public:

// Overrides
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	void InvertLine( CDC *dc, CPoint ptFrom, CPoint ptTo );

// Implementation
public:
	virtual ~CGraph4View();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:
	Network network;
	Algorithm algorithm;

	int LBdownNodeID;
	int LBupNodeID;
	int curr_node;
	int curr_start;
	int curr_end;
	double curr_weight;
	double curr_capacity;
	BOOL NodeHeld;

	CString curr_node_label;
	CPoint curr_point;
	CPoint startNodePoint;
	CPoint currLinePoint;
	CRect GraphingArea;
	CRect buttonArea;

// Generated message map functions
protected:

	DECLARE_MESSAGE_MAP()
public:
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnAlgorithmKruskal();
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnContextMenu(CWnd* /*pWnd*/, CPoint /*point*/);
	afx_msg void OnDeleteLink();
	afx_msg void OnDeleteLinkUpdate(CCmdUI *pCmdUI);
	afx_msg void OnLinkProps();
	afx_msg void OnAlgorithmRestore();
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnNodeProperties();
};

#ifndef _DEBUG  // debug version in Graph4View.cpp
inline CGraph4Doc* CGraph4View::GetDocument() const
   { return reinterpret_cast<CGraph4Doc*>(m_pDocument); }
#endif

