#include "stdafx.h"
#include "Network.h"
#include "Link.h"


#include <algorithm>
#include <vector>
#include "math.h"

int Network::counter = 1;

const double min_distance = 10.0;

Network::~Network()
{
}


//
// Add node to the current vector set
//
void Network::AddNode( CPoint point )
{
	Node n( point, counter++ );
	nodes.push_back( n );
}

//
// Add a link to the set
//
void Network::AddLink( int source, int target )
{
	Link addLink( source, target, 1 );
	links.push_back( addLink );
}


//
// Remove network link
//
void Network::RemoveLink( int source, int target )
{
	vector< Link >::iterator link_it;
	int count = 0;

	for ( link_it = links.begin();
		  link_it != links.end();
		  link_it++ )
	{
		if ( (*link_it).sourceNode == source &&
			 (*link_it).targetNode == target )
		{
			links.erase( links.begin() + count );
			break;
		}
		count++;
	}
}




//
// Plot the network nodes
//
void Network::PaintNodes(  )
{
	CClientDC dc( pWnd );
	CPen pen( PS_SOLID, 
			  1, 
			  RGB( 0, 0, 0 ) );

	CBrush brush( RGB( 0, 0, 0 ) );
	
	dc.SelectObject( pen );
	dc.SelectObject( brush );
			
	vector< Node >::iterator it;

	for ( it = nodes.begin();
		  it != nodes.end();
		  it++ )
	{
		
		int x = (*it).screenNode.x;
		int y = (*it).screenNode.y;

		// Draw the node
		dc.Ellipse( x - 5, y - 5, x + 5, y + 5 );	

		// Write the node text
		CString nodeTxt;
		nodeTxt.Format( /*"%i", (*it).*/(*it).label );		
		dc.TextOut( x, y - 20, nodeTxt );	
		
	}	
}


//
// Draw all network links
//
void Network::DrawLinks( vector< Link > lset, BOOL IsBlack  )
{	
	CClientDC dc( pWnd );	

	CPen blackPen( PS_SOLID, 
				   2, 
				   RGB( 0, 0, 0 ) );	

	CPen colouredPen( PS_SOLID, 
					  2, 
					  RGB( 0, 255, 0 ) );

	CPen whitePen( PS_SOLID, 
					  2, 
					  RGB( 255, 255, 255 ) );
	
	vector< Link >::iterator it;

	for( it = lset.begin();
		 it != lset.end();
		 it++ )
	{
		if ( (*it).linkColour == 0 )
			continue;

		CString lab = "";
		CPoint startPoint;
		CPoint targetPoint;
		int start  = -1;
		int target = -1;
		(*it).GetSourceTargetNodes( start, target );

		GetNodeData(  start,  startPoint, lab );
		GetNodeData( target, targetPoint, lab );

		// Get Pen colour
		CPen pen( PS_SOLID, 
				  2, 
				  (*it).linkColour == 0 ? RGB( 255, 255, 255 ) :
										  RGB( 0, 0, 0 ) );			

		dc.SelectObject( pen );
		dc.MoveTo( startPoint );
		dc.LineTo( targetPoint );
	}		
}


// 
// Does node exist within reasonable proximity of mouse click?
//
BOOL Network::NodeExists( CPoint point, int& nID )
{
	int NodeX = point.x;
	int NodeY = point.y;

	vector< Node >::iterator it;	

	for ( it = nodes.begin();
		  it != nodes.end();
		  it++ )
	{
		
		int X = (*it).screenNode.x;
		int Y = (*it).screenNode.y;

		double val = ( NodeX - X ) * ( NodeX - X ) + ( NodeY - Y ) * ( NodeY - Y );
		double distance = sqrt( val );

		if ( distance < min_distance ) 
		{
			nID = (*it).id;
			return TRUE;
		}
	}		

	return FALSE;
}


//
// Get coordinate of node
//
void Network::GetNodeData( int ID, CPoint& point, CString& label )
{
	vector< Node >::iterator it;		

	for ( it = nodes.begin();
		  it != nodes.end();
		  it++ )
	{
		
		if ( (*it).id == ID )
		{
			point.x = (*it).screenNode.x;
			point.y = (*it).screenNode.y;	
			label =   (*it).label;
			break;
		}		
	}		
}

// Etablish if link exists between two selected nodes
BOOL Network::LinkExists( int sourceID, int targetID )
{
	vector< Link >::iterator it;

	for ( it = links.begin();
		  it != links.end();
		  it++ )
	{

		int source = -1;
		int target = -1;
		(*it).GetSourceTargetNodes( source, target );

		if ( sourceID == source && targetID == target )
			return TRUE;
	}

	return FALSE;
}


// Returns the set of adjacent nodes
vector<int> Network::GetAdjNodes( int node )
{
	vector< int > adj;

	vector< Link >::iterator link_it;
	
	for ( link_it  = links.begin();
		  link_it != links.end();
		  link_it++ )
	{
		int start = -1;
		int target = -1;
		(*link_it).GetSourceTargetNodes( start, target );		

		if ( LinkExists( start, target ) )
		{
			if ( start == node || target == node )
			{
				adj.push_back( start == node ? target : start );
			}
		}
	}
				
	return adj;
}

//
// Set the properties of the selected link
//
void Network::SetLink( CPoint point )
{
	int start  = -1;
	int target = -1;
	double weight = 1.0;	

	GetLinkFromPoint( point, start, target, weight  );		
}


//
// Get actual (not screen) x,y coordinates of node
//
void Network::GetActualNodeCoordinate( int node, double& x, double& y )
{
	vector< Node >::iterator node_it;

	double x_val = -1.0;
	double y_val = -1.0;

	for ( node_it = nodes.begin();
		  node_it != nodes.end();
		  node_it++ )
	{
		if ( (*node_it).id == node )
		{
			x = (*node_it).actualNode.x;
			y = (*node_it).actualNode.y;
		}
	}	
}

//
// Get the link from the coordinate passed to it
// return TRUE if it exists
//
BOOL Network::GetLinkFromPoint( CPoint point, 
							   int& startNode, 
							   int& targetNode,
							   double& linkWeight )
{
	vector< Link >::iterator link_it;
	
	double MinDistance = 99999999999;

	for ( link_it  = links.begin();
		  link_it != links.end();
		  link_it++ )
	{
		// Get end points for this link
		int start  = -1;
		int target = -1;
		double wght   = 1;
		double cap    = 64;

		(*link_it).GetSourceTargetNodes( start, target );
		(*link_it).GetLinkWeight( wght );
		(*link_it).GetLinkCapacity( cap );

		// Get coordinate for these points if link exists
		if ( LinkExists( start, target ) )
		{
			double start_x  = -1.0;
			double start_y  = -1.0;
			double target_x = -1.0;
			double target_y = -1.0;

			GetActualNodeCoordinate( start,  start_x,  start_y  );
			GetActualNodeCoordinate( target, target_x, target_y );	

			// Get gradient and intercept of Line s-t
			double Mst = ( start_y - target_y ) / ( start_x - target_x );
			double Cst = start_y - Mst * start_x;

			// Get gradient and intercept of perpendicular
			double point_x = point.x;
			double point_y = point.y;

			double Mp = -1.0 / Mst;
			double Cp = point_y - Mp * point_x;

			// Get intersect point
			double intersect_x = - ( Cp - Cst ) / ( Mp - Mst );
			double intersect_y = Mp * ( - ( Cp - Cst ) / ( Mp - Mst ) ) + Cp;	
			
			// Get distance between mouse point and intersect point
			double distance = sqrt( ( intersect_x - point_x ) * ( intersect_x - point_x ) + 
									( intersect_y - point_y ) * ( intersect_y - point_y ) );

			if ( distance < MinDistance )
			{
				MinDistance = distance;
				startNode   = start;
				targetNode  = target;
				linkWeight  = wght;				
			}			
		}
	}
	
	return MinDistance <= 10.0;
}


//
// Get the link from the coordinate passed to it
// return TRUE if it exists
//
BOOL Network::LinkExistsAtPoint( CPoint point )
{
	vector< Link >::iterator link_it;
	
	double MinDistance = 99999999999;

	for ( link_it  = links.begin();
		  link_it != links.end();
		  link_it++ )
	{
		// Get end points for this link
		int start = -1;
		int target = -1;		

		(*link_it).GetSourceTargetNodes( start, target );
		
		// Get coordinate for these points if link exists
		if ( LinkExists( start, target ) )
		{
			double start_x  = -1.0;
			double start_y  = -1.0;
			double target_x = -1.0;
			double target_y = -1.0;

			GetActualNodeCoordinate( start,  start_x,  start_y  );
			GetActualNodeCoordinate( target, target_x, target_y );	

			// Get gradient and intercept of Line s-t
			double Mst = ( start_y - target_y ) / 
				         ( start_x - target_x );
			double Cst = start_y - Mst * start_x;

			// Get gradient and intercept of perpendicular
			double point_x = point.x;
			double point_y = point.y;

			double Mp = -1.0 / Mst;
			double Cp = point_y - Mp * point_x;

			// Get intersect point
			double intersect_x = - ( Cp - Cst ) / ( Mp - Mst );
			double intersect_y = Mp * ( - ( Cp - Cst ) / ( Mp - Mst ) ) + Cp;	
			
			// Get distance between mouse point and intersect point
			double distance = sqrt( ( intersect_x - point_x ) * ( intersect_x - point_x ) + 
									( intersect_y - point_y ) * ( intersect_y - point_y ) );

			if ( distance < MinDistance )
			{
				MinDistance = distance;					
			}				
		}
	}
	
	return MinDistance <= 10.0;
}


//
// Set the chosen node properties
//
void Network::SetNodeData( int node, CString label )
{
	vector< Node >::iterator node_it;

	for ( node_it = nodes.begin();
		  node_it != nodes.end();
		  node_it++ )
	{
		if ( (*node_it).id == node )
		{
			(*node_it).SetNodeLabel( label );
			break;
		}
	}
}

//
// Set the link weight value using start/end nodes passed
//
void Network::SetLinkWeight( int startNode, int targetNode, CString weight )
{
	vector< Link >::iterator link_it;

	for ( link_it  = links.begin();
		  link_it != links.end();
		  link_it++ )
	{
		// Get end points for this link
		int start = -1;
		int target = -1;
		(*link_it).GetSourceTargetNodes( start, target );

		// Set link weight if link exists
		if ( start == startNode && target == targetNode  )
		{			
			(*link_it).SetLinkWeight( atof( weight ) );
			break;
		}
	}	
}


//
// Colour the selected links
//
void Network::ColourLinks( vector< Link > &lset, int colour )
{
	for ( vector< Link >::iterator link_it = links.begin();
		  link_it != links.end();
		  link_it++ )
	{
		for ( vector< Link >::iterator lset_it = lset.begin();
			lset_it != lset.end();
			lset_it++ )
		{
			int link_start = (*link_it).sourceNode;
			int link_end   = (*link_it).targetNode;
			int lset_start = (*lset_it).sourceNode;
			int lset_end   = (*lset_it).targetNode;

			if ( link_start == lset_start &&
				 link_end   == lset_end )
			{
				(*link_it).linkColour = colour;
			}
		}
	}
}

//
// Reset all link colours to black
//
void Network::ResetLinkColours()
{
	for ( vector< Link >::iterator link_it = links.begin();
		  link_it != links.end();
		  link_it++ )
	{
		(*link_it).linkColour = 0;
	}
}

//
// Draw and update a line from start node to current cursor position 
// using 'rubber banding'
//
void Network::DrawLine( BOOL NodeHeld, CPoint startNodePoint, CPoint currLinePoint )
{	
	CClientDC dc( pWnd );		

	if ( NodeHeld )
	{
		CPen pen( PS_SOLID, 
				  2, 
				  RGB( 0, 0, 0 ) );			

		dc.SelectObject( pen );
		dc.MoveTo( startNodePoint );
		dc.LineTo( currLinePoint );		
	}
}